-- ------------------------------------------------------------------------------ --
--                                TradeSkillMaster                                --
--                          https://tradeskillmaster.com                          --
--    All Rights Reserved - Detailed license information included with addon.     --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...) ---@type TSM
local Operations = TSM:NewPackage("Operations") ---@type AddonPackage
local Operation = TSM.LibTSMTypes:Include("Operation")
local SessionInfo = TSM.LibTSMWoW:Include("Util.SessionInfo")
local Group = TSM.LibTSMTypes:Include("Group")
local TextureAtlas = TSM.Include("Util.TextureAtlas")
local private = {
	settingsDB = nil,
	settings = nil,
	characterKey = nil,
	shouldCreateDefaultOperations = false,
}



-- ============================================================================
-- Modules Functions
-- ============================================================================

function Operations.OnInitialize(settingsDB)
	local factionrealm = SessionInfo.GetFactionrealmName()
	private.characterKey = SessionInfo.GetCharacterName().." - "..factionrealm
	Operation.SetPlayerInfo(private.characterKey, factionrealm)
	private.settingsDB = settingsDB
	private.settings = settingsDB:NewView()
		:AddKey("global", "coreOptions", "globalOperations")
		:AddKey("global", "userData", "sharedOperations")
		:AddKey("profile", "userData", "operations")
		:AddKey("profile", "internalData", "createdDefaultOperations")
		:RegisterCallback("operations", private.OnProfileUpdated)
	private.LoadDBSettings()
	private.shouldCreateDefaultOperations = not private.settings.createdDefaultOperations
	private.settings.createdDefaultOperations = true
end

function Operations.RegisterType(operationType)
	Operation.RegisterType(operationType, private.shouldCreateDefaultOperations)
end

function Operations.GetFirstOperationByItem(moduleName, itemString)
	local groupPath = Group.GetPathByItem(itemString)
	for _, operationName in Group.OperationIterator(groupPath, moduleName) do
		Operation.UpdateFromRelationships(moduleName, operationName)
		if not Operation.IsIgnored(moduleName, operationName) then
			return operationName, Operation.GetSettings(moduleName, operationName)
		end
	end
end

function Operations.GetRelationshipColors(operationType, operationName, settingKey, value)
	local relationshipSet = Operation.HasRelationship(operationType, operationName, settingKey)
	local linkColor = nil
	if not value and relationshipSet then
		linkColor = "INDICATOR_DISABLED"
	elseif not value then
		linkColor = "TEXT_DISABLED"
	elseif relationshipSet then
		linkColor = "INDICATOR"
	else
		linkColor = "TEXT"
	end
	local linkTexture = TextureAtlas.GetColoredKey("iconPack.14x14/Link", linkColor)
	return relationshipSet, linkTexture, value and not relationshipSet and "TEXT" or "TEXT_DISABLED"
end

function Operations.SetStoredGlobally(storeGlobally)
	private.settings.globalOperations = storeGlobally
	if storeGlobally then
		-- Move the current profile operations to global
		private.settings.sharedOperations = CopyTable(private.settings.operations)
		-- Clear out old profile operations
		for _, profile in private.settingsDB:ScopeKeyIterator("profile") do
			private.settings:SetForScopeKey("operations", nil, profile)
		end
	else
		-- Copy global operations to all profiles
		for _, profile in private.settingsDB:ScopeKeyIterator("profile") do
			private.settings:SetForScopeKey("operations", CopyTable(private.settings.sharedOperations), profile)
		end
		-- Clear out old global operations
		private.settings.sharedOperations = nil
	end
	private.OnProfileUpdated()
end



-- ============================================================================
-- Private Helper Functions
-- ============================================================================

function private.OnProfileUpdated()
	private.LoadDBSettings()
	TSM.Groups.RebuildDatabase()
end

function private.LoadDBSettings()
	if private.settings.globalOperations then
		Operation.Load(private.settings.sharedOperations)
	else
		Operation.Load(private.settings.operations)
	end
end
