-- ------------------------------------------------------------------------------ --
--                                TradeSkillMaster                                --
--                          https://tradeskillmaster.com                          --
--    All Rights Reserved - Detailed license information included with addon.     --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...) ---@type TSM
local Buy = TSM.Vendoring:NewPackage("Buy")
local BagTracking = TSM.LibTSMService:Include("Inventory.BagTracking")
local Vendor = TSM.LibTSMService:Include("Vendor")



-- ============================================================================
-- Module Functions
-- ============================================================================

function Buy.GetMaxCanAfford(index)
	local row = Vendor.NewScannerQuery()
		:Equal("index", index)
		:GetSingleResultAndRelease()
	local maxCanAfford = math.huge

	-- Check the price
	local price = row:GetField("price")
	if price > 0 then
		maxCanAfford = min(floor(GetMoney() / price), maxCanAfford)
	end

	-- Check the additional cost
	for _, itemString, quantity in Vendor.AdditionalCostIterator(index) do
		local currencyId = strmatch(itemString, "currency:(%d+)")
		local numHave = nil
		if currencyId then
			numHave = C_CurrencyInfo.GetCurrencyInfo(currencyId).quantity
		else
			numHave = BagTracking.GetTotalQuantity(itemString)
		end
		maxCanAfford = min(floor(numHave / quantity), maxCanAfford)
	end

	local stackSize = row:GetField("stackSize")
	row:Release()
	return maxCanAfford * stackSize
end

function Buy.BuyItem(itemString, quantity)
	local index = Vendor.GetFirstIndex(itemString)
	if not index then
		return
	end
	Buy.BuyItemIndex(index, quantity)
end

function Buy.BuyItemIndex(index, quantity)
	quantity = min(quantity, Buy.GetMaxCanAfford(index))
	if quantity == 0 then
		return
	end
	Vendor.BuyIndex(index, quantity)
end
