-- ------------------------------------------------------------------------------ --
--                                TradeSkillMaster                                --
--                          https://tradeskillmaster.com                          --
--    All Rights Reserved - Detailed license information included with addon.     --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...) ---@type TSM
local ProfessionInfo = TSM.Init("Data.ProfessionInfo") ---@class Data.ProfessionInfo
local ClientInfo = TSM.LibTSMWoW:Include("Util.ClientInfo")
local EnchantData = TSM.LibTSMData:Include("Enchant")
local OptionalMatData = TSM.LibTSMData:Include("OptionalMat")
local SalvageData = TSM.LibTSMData:Include("Salvage")



-- ============================================================================
-- Profession Info Data
-- ============================================================================

local PROFESSION_NAMES = {
	Mining = GetSpellInfo(2575),
	Smelting = GetSpellInfo(2656),
	Poisons = GetSpellInfo(2842),
}
local CLASSIC_SUB_NAMES = {
	[APPRENTICE] = true,
	[JOURNEYMAN] = true,
	[EXPERT] = true,
	[ARTISAN] = true,
	[MASTER] = true,
	[GRAND_MASTER] = true,
	["大师级"] = true, -- zhCN ARTISAN
	["Мастеровой"] = true, -- ruRU ARTISAN
}
local PROFESSION_NAME_MAP = {
	["Costura"] = "Sastrería",
	["Marroquinería"] = "Peletería",
	["Ingénierie"] = "Ingénieur",
	["Secourisme"] = "Premiers soins",
}



-- ============================================================================
-- Module Functions
-- ============================================================================

function ProfessionInfo.GetName(key)
	local name = PROFESSION_NAMES[key]
	assert(name)
	return name
end

function ProfessionInfo.IsSubNameClassic(str)
	assert(ClientInfo.HasFeature(ClientInfo.FEATURES.SUB_PROFESSION_NAMES))
	return CLASSIC_SUB_NAMES[str] or false
end

function ProfessionInfo.MapProfessionName(str)
	return PROFESSION_NAME_MAP[str]
end

function ProfessionInfo.GetIndirectCraftResult(spellId)
	return EnchantData.Recipes[spellId] or SalvageData.MassMill[spellId] or EnchantData.QualityRecipes[spellId] or nil
end

function ProfessionInfo.GetOptionalMatByItemLevel(itemLevel)
	for itemString, info in pairs(OptionalMatData.Info) do
		if info.absItemLevel == itemLevel then
			return itemString
		end
	end
	return nil
end

function ProfessionInfo.GetOptionalMatByRelItemLevel(relItemLevel)
	for itemString, info in pairs(OptionalMatData.Info) do
		if info.relItemLevels and info.relItemLevels[relItemLevel] and not info.ignored then
			return itemString
		end
	end
	return nil
end

function ProfessionInfo.GetRequiredLevelByOptionalMat(itemString)
	local info = OptionalMatData.Info[itemString]
	return info and info.reqCraftLevels or nil
end

function ProfessionInfo.GetItemLevelByOptionalMat(itemString)
	local info = OptionalMatData.Info[itemString]
	return info and info.absItemLevel or nil
end

function ProfessionInfo.GetCraftLevelIncreaseByOptionalMat(itemString)
	local info = OptionalMatData.Info[itemString]
	return info and info.relCraftLevel or nil
end

function ProfessionInfo.GetOptionalMatByStatModifier(statModifier)
	for itemString, info in pairs(OptionalMatData.Info) do
		if info.statModifier == statModifier then
			return itemString
		end
	end
	return nil
end
