-- ------------------------------------------------------------------------------ --
--                                TradeSkillMaster                                --
--                          https://tradeskillmaster.com                          --
--    All Rights Reserved - Detailed license information included with addon.     --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...)
local ChatMessage = TSM.Init("Util.ChatMessage")
local Theme = TSM.Include("Util.Theme")
local private = {
	chatFrame = nil,
}



-- ============================================================================
-- Module Functions
-- ============================================================================

---Sets the chat frame to display messages within.
---@param chatFrame any
function ChatMessage.SetChatFrame(chatFrame)
	private.chatFrame = strlower(chatFrame)
end

---Prints a raw user-facing message to chat.
---@param str string The message
function ChatMessage.PrintUserRaw(str)
	private.GetChatFrame():AddMessage(str)
end

---Prints a raw, formatted user-facing message to chat.
---@param ... string The message
function ChatMessage.PrintfUserRaw(...)
	ChatMessage.PrintUserRaw(format(...))
end

---Prints a user-facing message to chat.
---@param str string The message
function ChatMessage.PrintUser(str)
	ChatMessage.PrintUserRaw(Theme.GetColor("INDICATOR"):ColorText("TSM")..": "..str)
end

---Prints a formatted user-facing message to chat.
---@param ... string The message
function ChatMessage.PrintfUser(...)
	ChatMessage.PrintUser(format(...))
end

---Colors some accent text for display to the user.
---@param text string The text to color
---@return string @The colored text
function ChatMessage.ColorUserAccentText(text)
	return Theme.GetColor("INDICATOR_ALT"):ColorText(text)
end



-- ============================================================================
-- Private Helper Functions
-- ============================================================================

function private.GetChatFrame()
	for i = 1, NUM_CHAT_WINDOWS do
		local name = strlower(GetChatWindowInfo(i) or "")
		if name ~= "" and name == private.chatFrame then
			return _G["ChatFrame" .. i]
		end
	end
	return DEFAULT_CHAT_FRAME
end
